<?php
/**
  Plugin Name: Subscribe to Category
  Plugin URI:
  Description: Lets your visitor subscribe to posts for one or several categories.
  Version: 2.4.2
  Author: Daniel Söderström / Sidney van de Stouwe
  Author URI: https://vandestouwe.com
  License: GPLv2 or later
  Text Domain: subscribe-to-category
  Domain Path: /languages
 *
 * @package subscribe-to-category
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die();
}

define( 'STC_VERSION', '2.4.2' );
define( 'STC_SLUG', 'stc' );
define( 'STC_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
define( 'STC_PLUGIN_PATH', dirname( __FILE__ ) );
define( 'STC_DEV_MODE', false ); // set to true email is printed out on setting page and will not be sent.

// pickup the cron reschedule time if it was set by the user else set it to 1 hour
$cron_time = get_option( 'stc_settings' )['cron_time'];
if (!isset($cron_time)) $cron_time = 3600;

require_once( 'classes/class-stc-main.php' );
require_once( 'classes/class-stc-settings.php' );
require_once( 'classes/class-stc-subscribe.php' );
require_once( 'classes/class-stc-widget.php' );

// Create instance for main class.
add_action( 'plugins_loaded', array( 'STC_Main', 'get_instance' ) );

// Create a custom event in WP-Cron
add_filter( 'cron_schedules', 'wpshout_add_cron_interval' );
function wpshout_add_cron_interval( $schedules ) {
    global $cron_time;    
    $schedules['stc_reschedule_time'] = array(
            'interval'  => $cron_time, // reschedule time in seconds
            'display'   => 'STC Reschedule Time'
    );
    return $schedules;
}

function general_admin_notice(){
    global $pagenow;
    global $message;
    global $severity;
    if ( $pagenow == 'options-general.php' ) {
         echo '<div class="notice '. $severity .' " is-dismissible">
             <p><strong>'. $message.'</p>
         </div>';
    }
}

// Register activation hook.
register_activation_hook( __FILE__, array( 'STC_Main', 'activate' ) );

// Register deactivation hook.
register_deactivation_hook( __FILE__, array( 'STC_Main', 'deactivate' ) );

register_post_meta(
	'',
	'_stc_notifier_status',
	array(
		'show_in_rest' => true,
		'single' => true,
		'type' => 'string',
		'auth_callback' => function() {
			return current_user_can( 'edit_posts' );
		},
	)
);

register_post_meta(
	'',
	'_stc_notifier_sent_time',
	array(
		'show_in_rest' => true,
		'single' => true,
		'type' => 'string',
		'auth_callback' => function() {
			return current_user_can( 'edit_posts' );
		},
	)
);
register_post_meta(
	'',
	'_stc_notifier_request',
	array(
		'type' => 'boolean',
		'show_in_rest' => true,
		'single' => true,
		'auth_callback' => function() {
			return current_user_can( 'edit_posts' );
		},
	)
);


